@@ -6,13 +6,14 @@ import requests
 from PIL import Image, ImageDraw, ExifTags
 from multiprocessing.pool import ThreadPool
 from multiprocessing import Pool, Value
+import picsellia_training.pxl_multithreading as mlt
 import time
 import zipfile
-import pxl_exceptions as exceptions
+import picsellia_training.pxl_exceptions as exceptions
 from uuid import UUID, uuid4
-from pxl_utils import is_uuid
-import pxl_utils as utils
-from pxl_urls_v2 import Urls as urls
+from picsellia_training.pxl_utils import is_uuid
+import picsellia_training.pxl_utils as utils
+from picsellia_training.pxl_urls_v2 import Urls as urls
 from pathlib import Path
 
 class Client:
@@ -124,7 +125,7 @@ class Client:
                     identifier = name
             if identifier == None:
                 raise Exception('No corresponding experiment found, please enter a correct experiment id or a correct experiment name + project token')
-            experiment = self.get(identifier, with_file=with_file, with_data=with_data)["experiment"][0]
+            experiment = self.get(identifier, with_file=with_file, with_data=with_data)
             self.files = experiment["files"]
             self.data = experiment["data"]
             self.id = experiment["id"]
@@ -147,7 +148,7 @@ class Client:
                             if name == 'config':
                                 self.dl_file(object_name, os.path.join(self.config_dir, filename))
                             elif name == 'checkpoint-index-latest':
-                                self.dl_file(object_name, os.path.join(self.exported_model_dir, filename))
+                                self.dl_file(object_name, os.path.join(self.checkpoint_dir, filename))
                             else:
                                 self.dl_file(object_name, os.path.join(self.base_dir, filename))
             else:
@@ -601,7 +602,7 @@ class Client:
                 raise exceptions.AuthenticationError(r.json()["error"])
             return True
         
-        def update_data(self, data_name, id=None, **kwargs):
+        def update_data(self, data_name=None, id=None, **kwargs):
             """[summary]
 
             Args:
@@ -619,10 +620,10 @@ class Client:
             assert self.id != None or id != None, "Please checkout an experiment or enter the desired experiment id"
             if self.id != None:
                 id = self.id
-            assert name != None, "Please enter a valid data asset name"
+            assert data_name != None, "Please enter a valid data asset name"
             data = json.dumps(kwargs)
             try:
-                r = requests.patch(self.host + 'experiment/{}/data/{}'.format(id, name), data=data, headers=self.auth)
+                r = requests.patch(self.host + 'experiment/{}/data/{}'.format(id, data_name), data=data, headers=self.auth)
             except Exception:
                 raise exceptions.NetworkError(r.json()["error"])
             
@@ -640,7 +641,7 @@ class Client:
             parts = self.urls._upload_part(path, object_name)
 
             if self.urls._complete_part_upload(parts, object_name, None):
-                self._create_or_update_file(name, path, id, object_name, True)
+                self._create_or_update_file(name, path, id, object_name=object_name, large=True)
 
         def _send_file(self, path=None, name=None, object_name=None, network_id=None, id=None):
             error_message = "Please checkout an experiment/network or enter the desired experiment id/network_id"
@@ -653,7 +654,7 @@ class Client:
                     files = {'file': (path, f)}
                     http_response = requests.post(response['url'], data=response['fields'], files=files)
                 if http_response.status_code == 204:
-                    self._create_or_update_file(name, path, id, object_name, False)
+                    self._create_or_update_file(name, path, id, object_name=object_name, large=False)
             except Exception as e:
                 raise exceptions.NetworkError(str(e))
 
@@ -666,18 +667,18 @@ class Client:
                 self.create_data(id, name, data)
             else:
                 asset = stored[0]
-                self.update_data(id, asset["id"], name, name, data)
+                self.update_data(name, id, data=data)
             
-        def _create_or_update_file(self, name="", path="", id=None, object_name=None, large=False):
+        def _create_or_update_file(self, file_name="", path="", id=None, **kwargs):
             assert self.id != None or id != None, "Please checkout an experiment or enter the desired experiment id"
             if self.id != None:
                 id = self.id
-            stored = self.get_file(id, name)
+            stored = self.get_file(id, file_name)
             if stored == []:
-                self.create_file(id, name, object_name, large)
+                self.create_file(id, file_name, object_name, large)
             else:
                 asset = stored[0]
-                self.update_file(id, asset["id"], name, name, object_name, large)
+                self.update_file(file_name=file_name, id=id, **kwargs)
 
         def store(self, name="", path=None, id=None, zip=False):
             assert self.id != None or id != None, "Please checkout an experiment or enter the desired experiment id"
@@ -792,6 +793,146 @@ class Client:
             if not os.path.isdir(dir_name):
                 os.mkdir(dir_name)
 
+        def dl_annotations(self, option="all"):
+            """ Download all the annotations made on Picsell.ia Platform for your project.
+            Called when checking out a network
+            Args:
+                option (str): Define what type of annotation to export (accepted or all)
+
+            Raises:
+                NetworkError: If Picsell.ia server is not responding or host is incorrect.
+                ResourceNotFoundError: If we can't find any annotations for that project.
+            """
+
+            print("Downloading annotations ...")
+
+            try:
+                to_send = {"project_token": self.project_token, "type": option}
+                r = requests.get(self.host + 'download_annotations', data=json.dumps(to_send), headers=self.auth)
+            except Exception:
+                raise exceptions.NetworkError("Server is not responding, please check your host or Picsell.ia server status on twitter")
+            if r.status_code != 200:
+                raise exceptions.ResourceNotFoundError("No annotations were found for this project")
+
+            self.dict_annotations = r.json()
+
+            if len(self.dict_annotations.keys()) == 0:
+                raise exceptions.ResourceNotFoundError("You don't have any annotations")
+
+        def dl_pictures(self):
+            """Download your training set on the machine (Use it to dl images to Google Colab etc.)
+            Save it to /project_id/images/*
+            Perform train_test_split & send the repartition to Picsell.ia Platform
+
+            Raises:
+                ResourceNotFoundError : If no annotations in the Picsell.ia Client yet or images can't be downloaded"""
+
+            if not hasattr(self, "dict_annotations"):
+                raise exceptions.ResourceNotFoundError("Please dl_annotations model with dl_annotations()")
+
+            if "images" not in self.dict_annotations.keys():
+                raise exceptions.ResourceNotFoundError("Please run dl_annotations function first")
+
+            print("Downloading images ...")
+
+            if not os.path.isdir(self.png_dir):
+                os.makedirs(self.png_dir)
+
+            lst = []
+            for info in self.dict_annotations["images"]:
+                lst.append(info["external_picture_url"])
+            t = len(set(lst))
+            print('-----')
+            nb_threads = 20
+            infos_split = list(mlt.chunks(self.dict_annotations["images"], nb_threads))
+            counter = Value('i', 0)
+            p = Pool(nb_threads, initializer=mlt.pool_init, 
+                initargs=(t, self.png_dir, counter,self.interactive,))
+            p.map(mlt.dl_list, infos_split)
+            print('--*--')
+            print("Images downloaded")
+
+        def generate_labelmap(self):
+            """THIS FUNCTION IS MAINTAINED FOR TENSORFLOW 1.X
+            ----------------------------------------------------------
+            Genrate the labelmap.pbtxt file needed for Tensorflow training at:
+                - project_id/
+                    network_id/
+                        training_id/
+                            label_map.pbtxt
+            Raises:
+                ResourceNotFoundError : If no annotations in the Picsell.ia Client yet or images can't be downloaded
+                                        If no directories have been created first."""
+
+            print("Generating labelmap ...")
+            if not hasattr(self, "dict_annotations") or not hasattr(self, "base_dir"):
+                raise exceptions.ResourceNotFoundError("Please run create_network() or checkout_network() then dl_annotations()")
+
+            self.label_path = os.path.join(self.base_dir, "label_map.pbtxt")
+
+            if "categories" not in self.dict_annotations.keys():
+                raise exceptions.ResourceNotFoundError("Please run dl_annotations() first")
+
+            categories = self.dict_annotations["categories"]
+            labels_Network = {}
+            try:
+                with open(self.label_path, "w+") as labelmap_file:
+                    for k, category in enumerate(categories):
+                        name = category["name"]
+                        labelmap_file.write("item {\n\tname: \"" + name + "\"" + "\n\tid: " + str(k + 1) + "\n}\n")
+                        # if self.project_type == 'classification':
+                        #     labels_Network[str(k)] = name
+                        # else:
+                        labels_Network[str(k + 1)] = name
+                    labelmap_file.close()
+                print(f"Label_map.pbtxt created @ {self.label_path}")
+
+            except Exception:
+                raise exceptions.ResourceNotFoundError("No directory found, please call checkout_network() or create_network() function first")
+
+            self.label_map = labels_Network
+
+        def train_test_split(self, prop=0.8):
+
+            if not hasattr(self, "dict_annotations"):
+                raise exceptions.ResourceNotFoundError("Please download annotations first")
+
+            if "images" not in self.dict_annotations.keys():
+                raise exceptions.ResourceNotFoundError("Please download annotations first")
+
+            self.train_list = []
+            self.eval_list = []
+            self.train_list_id = []
+            self.eval_list_id = []
+            self.index_url = utils.train_valid_split_obj_simple(self.dict_annotations, prop)
+
+            total_length = len(self.dict_annotations["images"])
+            for info, idx in zip(self.dict_annotations["images"], self.index_url):
+                pic_name = os.path.join(self.png_dir, info['external_picture_url'])
+                if idx == 1:
+                    self.train_list.append(pic_name)
+                    self.train_list_id.append(info["internal_picture_id"])
+                else:
+                    self.eval_list.append(pic_name)
+                    self.eval_list_id.append(info["internal_picture_id"])
+
+            print(f"{len(self.train_list_id)} images used for training and {len(self.eval_list_id)} images used for validation")
+
+            label_train, label_test, cate = utils.get_labels_repartition_obj_detection(self.dict_annotations, self.index_url)
+
+            # to_send = {"project_token": self.project_token,
+            #         "train": {"train_list_id": self.train_list_id, "label_repartition": label_train, "labels": cate},
+            #         "eval": {"eval_list_id": self.eval_list_id, "label_repartition": label_test, "labels": cate},
+            #         "network_id": self.network_id, "training_id": self.training_id}
+
+            # try:
+            #     r = requests.post(self.host + 'post_repartition', data=json.dumps(to_send), headers=self.auth)
+            #     if r.status_code != 201:
+            #         raise exceptions.NetworkError('Can not send repartition to Picsell.ia Backend')
+            #     print("Repartition sent ..")
+            # except Exception:
+            #     raise exceptions.NetworkError('Can not send repartition to Picsell.ia Backend')
+
     class Dataset:
 
         def __init__(self, host, auth, dataset_id=None):
